@@ -36,15 +36,15 @@ static void do_test_u32_varint(uint32_t nr, int expected_size, char* expected_re
 	int size;
 	char *hex;
 	uint8_t buf[16];
-	
+
 	size = varint_u32_encode_buf(nr, buf);
 	if ( size != expected_size )
 		printf("Expected: %d\nObtained: %d\n", expected_size, size);
 
 	CU_ASSERT_EQUAL(size, expected_size);
-	
+
 	hex = hexbytes_from_bytes(buf, size);
-	ASSERT_STRING_EQUAL(hex, expected_res);	
+	ASSERT_STRING_EQUAL(hex, expected_res);
 	lwfree(hex);
 }
 
@@ -53,7 +53,7 @@ static void do_test_s32_varint(int32_t nr,int expected_size, char* expected_res)
 	uint8_t buf[16];
 	int size;
 	char *hex;
-	
+
 	size = varint_s32_encode_buf(nr, buf);
 	if ( size != expected_size )
 	{
@@ -62,7 +62,7 @@ static void do_test_s32_varint(int32_t nr,int expected_size, char* expected_res)
 	CU_ASSERT_EQUAL(size,expected_size);
 
 	hex = hexbytes_from_bytes(buf, size);
-	ASSERT_STRING_EQUAL(hex, expected_res);	
+	ASSERT_STRING_EQUAL(hex, expected_res);
 	lwfree(hex);
 }
 
@@ -71,7 +71,7 @@ static void do_test_u64_varint(uint64_t nr,int expected_size, char* expected_res
 	uint8_t buf[16];
 	int size;
 	char *hex;
-	
+
 	size = varint_u64_encode_buf(nr, buf);
 	if ( size != expected_size )
 	{
@@ -89,16 +89,16 @@ static void do_test_s64_varint(int64_t nr,int expected_size, char* expected_res)
 	uint8_t buf[16];
 	int size;
 	char *hex;
-	
+
 	size = varint_s64_encode_buf(nr, buf);
 	if ( size != expected_size )
 	{
 		printf("Expected: %d\nObtained: %d\n", expected_size, size);
 	}
 	CU_ASSERT_EQUAL(size,expected_size);
-	
+
 	hex = hexbytes_from_bytes(buf,size);
-	ASSERT_STRING_EQUAL(hex, expected_res);	
+	ASSERT_STRING_EQUAL(hex, expected_res);
 	lwfree(hex);
 }
 
@@ -198,6 +198,9 @@ static void do_test_s64_roundtrip(int64_t i64_in)
 
 static void test_varint_roundtrip(void)
 {
+
+	do_test_u64_roundtrip(0xFFFFFFFFFFFFFFFF);
+
 	int i;
 	for ( i = 0; i < 1024; i += 63 )
 	{
@@ -218,7 +221,7 @@ static void test_zigzag(void)
 		a = b = i;
 		CU_ASSERT_EQUAL(a, unzigzag64(zigzag64(a)));
 		CU_ASSERT_EQUAL(b, unzigzag32(zigzag64(b)));
-		
+
 		a = b = -1 * i;
 		CU_ASSERT_EQUAL(a, unzigzag64(zigzag64(a)));
 		CU_ASSERT_EQUAL(b, unzigzag32(zigzag64(b)));
