@@ -25,10 +25,16 @@
 
 #include "liblwgeom_internal.h"
 #include "lwgeom_log.h"
+#include "lwgeodetic.h"
 
 /***********************************************************************
 * GSERIALIZED metadata utility functions.
 */
+/* handle missaligned uint32_t data */
+static inline uint32_t gserialized_get_uint32_t(const uint8_t *loc)
+{
+	return *((uint32_t*)loc);
+}
 
 int gserialized_has_bbox(const GSERIALIZED *gser)
 {
@@ -66,10 +72,19 @@ uint32_t gserialized_max_header_size(void)
 	return sizeof(GSERIALIZED) + 8 * sizeof(float) + sizeof(int);
 }
 
+uint32_t gserialized_header_size(const GSERIALIZED *gser)
+{
+	uint32_t sz = 8; /* varsize (4) + srid(3) + flags (1) */
+
+	if (gserialized_has_bbox(gser))
+		sz += gbox_serialized_size(gser->flags);
+
+	return sz;
+}
+
 uint32_t gserialized_get_type(const GSERIALIZED *s)
 {
 	uint32_t *ptr;
-	assert(s);
 	ptr = (uint32_t*)(s->data);
 	LWDEBUG(4,"entered");
 	if ( FLAGS_GET_BBOX(s->flags) )
@@ -89,7 +104,7 @@ int32_t gserialized_get_srid(const GSERIALIZED *s)
 	/* Only the first 21 bits are set. Slide up and back to pull
 	   the negative bits down, if we need them. */
 	srid = (srid<<11)>>11;
-	
+
 	/* 0 is our internal unknown value. We'll map back and forth here for now */
 	if ( srid == 0 )
 		return SRID_UNKNOWN;
@@ -107,7 +122,7 @@ void gserialized_set_srid(GSERIALIZED *s, int32_t srid)
 	 * We'll map back and forth here for now */
 	if ( srid == SRID_UNKNOWN )
 		srid = 0;
-		
+
 	s->srid[0] = (srid & 0x001F0000) >> 16;
 	s->srid[1] = (srid & 0x0000FF00) >> 8;
 	s->srid[2] = (srid & 0x000000FF);
@@ -130,7 +145,7 @@ static size_t gserialized_is_empty_recurse(const uint8_t *p, int *isempty)
 
 	memcpy(&type, p, 4);
 	memcpy(&num, p+4, 4);
-	
+
 	if ( lwtype_is_collection(type) )
 	{
 		size_t lz = 8;
@@ -169,6 +184,187 @@ char* gserialized_to_string(const GSERIALIZED *g)
 	return lwgeom_to_wkt(lwgeom_from_gserialized(g), WKT_ISO, 12, 0);
 }
 
+/* Unfortunately including advanced instructions is something that
+only helps a small sliver of users who can build their own
+knowing the target system they will be running on. Packagers
+have to aim for the lowest common demoninator. So this is
+dead code for the forseeable future. */
+#define HAVE_PDEP 0
+#if HAVE_PDEP
+/* http://www.joshbarczak.com/blog/?p=454 */
+static uint64_t uint32_interleave_2(uint32_t u1, uint32_t u2)
+{
+    uint64_t x = u1;
+    uint64_t y = u2;
+    uint64_t x_mask = 0x5555555555555555;
+    uint64_t y_mask = 0xAAAAAAAAAAAAAAAA;
+    return _pdep_u64(x, x_mask) | _pdep_u64(y, y_mask);
+}
+
+static uint64_t uint32_interleave_3(uint32_t u1, uint32_t u2, uint32_t u3)
+{
+    /* only look at the first 21 bits */
+    uint64_t x = u1 & 0x1FFFFF;
+    uint64_t y = u2 & 0x1FFFFF;
+    uint64_t z = u3 & 0x1FFFFF;
+    uint64_t x_mask = 0x9249249249249249;
+    uint64_t y_mask = 0x2492492492492492;
+    uint64_t z_mask = 0x4924924924924924;
+    return _pdep_u64(x, x_mask) | _pdep_u64(y, y_mask) | _pdep_u64(z, z_mask);
+}
+
+#else
+static uint64_t uint32_interleave_2(uint32_t u1, uint32_t u2)
+{
+    uint64_t x = u1;
+    uint64_t y = u2;
+    int i;
+
+    static uint64_t B[5] =
+    {
+        0x5555555555555555,
+        0x3333333333333333,
+        0x0F0F0F0F0F0F0F0F,
+        0x00FF00FF00FF00FF,
+        0x0000FFFF0000FFFF
+    };
+    static uint64_t S[5] = { 1, 2, 4, 8, 16 };
+
+    for ( i = 4; i >= 0; i-- )
+    {
+        x = (x | (x << S[i])) & B[i];
+        y = (y | (y << S[i])) & B[i];
+    }
+
+    return x | (y << 1);
+}
+#endif
+
+
+uint64_t gbox_get_sortable_hash(const GBOX *g)
+{
+	uint32_t ux, uy;
+	float fx, fy;
+
+	/*
+	* Since in theory the bitwise representation of an IEEE
+	* float is sortable (exponents come before mantissa, etc)
+	* we just copy the bits directly into an int and then
+	* interleave those ints.
+	*/
+	if ( FLAGS_GET_GEODETIC(g->flags) )
+	{
+		GEOGRAPHIC_POINT gpt;
+		POINT3D p;
+		p.x = (g->xmax + g->xmin) / 2.0;
+		p.y = (g->ymax + g->ymin) / 2.0;
+		p.z = (g->zmax + g->zmin) / 2.0;
+		normalize(&p);
+		cart2geog(&p, &gpt);
+		fx = gpt.lon;
+		fy = gpt.lat;
+		memcpy(&ux, &fx, sizeof(uint32_t));
+		memcpy(&uy, &fy, sizeof(uint32_t));
+	}
+	else
+	{
+		fx = (g->xmax + g->xmin) / 2.0;
+		fy = (g->ymax + g->ymin) / 2.0;
+		memcpy(&ux, &fx, sizeof(uint32_t));
+		memcpy(&uy, &fy, sizeof(uint32_t));
+	}
+	return uint32_interleave_2(ux, uy);
+}
+
+int gserialized_cmp(const GSERIALIZED *g1, const GSERIALIZED *g2)
+{
+	int g1_is_empty, g2_is_empty, cmp;
+	GBOX box1, box2;
+	size_t sz1 = SIZE_GET(g1->size);
+	size_t sz2 = SIZE_GET(g2->size);
+	size_t hsz1 = gserialized_header_size(g1);
+	size_t hsz2 = gserialized_header_size(g2);
+
+	uint8_t *b1 = (uint8_t*)g1 + hsz1;
+	uint8_t *b2 = (uint8_t*)g2 + hsz2;
+	size_t bsz1 = sz1 - hsz1;
+	size_t bsz2 = sz2 - hsz2;
+	size_t bsz = bsz1 < bsz2 ? bsz1 : bsz2;
+
+	uint64_t hash1, hash2;
+	int32_t srid1 = gserialized_get_srid(g1);
+	int32_t srid2 = gserialized_get_srid(g2);
+
+	g1_is_empty = (gserialized_get_gbox_p(g1, &box1) == LW_FAILURE);
+	g2_is_empty = (gserialized_get_gbox_p(g2, &box2) == LW_FAILURE);
+
+	/* Empty == Empty */
+	if (g1_is_empty && g2_is_empty)
+    {
+        /* POINT EMPTY == POINT EMPTY */
+        /* POINT EMPTY < LINESTRING EMPTY */
+        uint32_t t1 = gserialized_get_type(g1);
+        uint32_t t2 = gserialized_get_type(g2);
+		return t1 == t2 ? 0 : (t1 < t2 ? -1 : 1);
+    }
+
+	/* Empty < Non-empty */
+	if (g1_is_empty)
+		return -1;
+
+	/* Non-empty > Empty */
+	if (g2_is_empty)
+		return 1;
+
+	/* Return equality for perfect equality only */
+	cmp = memcmp(b1, b2, bsz);
+	if ( bsz1 == bsz2 && srid1 == srid2 && cmp == 0 )
+		return 0;
+
+	/* Using the centroids, calculate somewhat sortable */
+    /* hash key. The key doesn't provide good locality over */
+    /* the +/- boundary, but otherwise is pretty OK */
+	hash1 = gbox_get_sortable_hash(&box1);
+	hash2 = gbox_get_sortable_hash(&box2);
+
+	if ( hash1 > hash2 )
+		return 1;
+	else if ( hash1 < hash2 )
+		return -1;
+
+	/* What, the hashes are equal? OK... sort on the */
+	/* box minima */
+	if (box1.xmin < box2.xmin)
+		return -1;
+	else if (box1.xmin > box2.xmin)
+		return 1;
+
+	if (box1.ymin < box2.ymin)
+		return -1;
+	else if (box1.ymin > box2.ymin)
+		return 1;
+
+	/* Still equal? OK... sort on the box maxima */
+	if (box1.xmax < box2.xmax)
+		return -1;
+	else if (box1.xmax > box2.xmax)
+		return 1;
+
+	if (box1.ymax < box2.ymax)
+		return -1;
+	else if (box1.ymax > box2.ymax)
+		return 1;
+
+	/* Geeze! How about object size? Sort on that... */
+	if (hsz1 < hsz2)
+		return -1;
+	else if (hsz1 > hsz2)
+		return 1;
+
+	/* OK fine, we'll sort on the memcmp just to be done with this */
+	return cmp == 0 ? 0 : (cmp > 0 ? 1 : -1);
+}
+
 int gserialized_read_gbox_p(const GSERIALIZED *g, GBOX *gbox)
 {
 
@@ -225,7 +421,7 @@ static int gserialized_peek_gbox_p(const GSERIALIZED *g, GBOX *gbox)
 	{
 		return LW_FAILURE;
 	}
-	
+
 	/* Boxes of points are easy peasy */
 	if ( type == POINTTYPE )
 	{
@@ -261,22 +457,22 @@ static int gserialized_peek_gbox_p(const GSERIALIZED *g, GBOX *gbox)
 		double *dptr = (double*)(g->data);
 		int *iptr = (int*)(g->data);
 		int npoints = iptr[1]; /* Read the npoints */
-	
+
 		/* This only works with 2-point lines */
 		if ( npoints != 2 )
 			return LW_FAILURE;
-		
+
 		/* Advance to X */
 		/* Past <linetype><npoints> */
 		i++;
 		gbox->xmin = FP_MIN(dptr[i], dptr[i+ndims]);
 		gbox->xmax = FP_MAX(dptr[i], dptr[i+ndims]);
-	
+
 		/* Advance to Y */
 		i++;
 		gbox->ymin = FP_MIN(dptr[i], dptr[i+ndims]);
 		gbox->ymax = FP_MAX(dptr[i], dptr[i+ndims]);
-	
+
 		gbox->flags = g->flags;
 		if ( FLAGS_GET_Z(g->flags) )
 		{
@@ -363,12 +559,12 @@ static int gserialized_peek_gbox_p(const GSERIALIZED *g, GBOX *gbox)
 		i += 2;
 		gbox->xmin = FP_MIN(dptr[i], dptr[i+ndims]);
 		gbox->xmax = FP_MAX(dptr[i], dptr[i+ndims]);
-	
+
 		/* Advance to Y */
 		i++;
 		gbox->ymin = FP_MIN(dptr[i], dptr[i+ndims]);
 		gbox->ymax = FP_MAX(dptr[i], dptr[i+ndims]);
-	
+
 		gbox->flags = g->flags;
 		if ( FLAGS_GET_Z(g->flags) )
 		{
@@ -387,7 +583,7 @@ static int gserialized_peek_gbox_p(const GSERIALIZED *g, GBOX *gbox)
 		gbox_float_round(gbox);
 		return LW_SUCCESS;
 	}
-	
+
 	return LW_FAILURE;
 }
 
@@ -567,13 +763,13 @@ size_t gserialized_from_lwgeom_size(const LWGEOM *geom)
 {
 	size_t size = 8; /* Header overhead. */
 	assert(geom);
-	
+
 	if ( geom->bbox )
-		size += gbox_serialized_size(geom->flags);	
-		
+		size += gbox_serialized_size(geom->flags);
+
 	size += gserialized_from_any_size(geom);
 	LWDEBUGF(3, "g_serialize size = %d", size);
-	
+
 	return size;
 }
 
@@ -946,7 +1142,7 @@ GSERIALIZED* gserialized_from_lwgeom(LWGEOM *geom, size_t *size)
 	{
 		lwgeom_add_bbox(geom);
 	}
-	
+
 	/*
 	** Harmonize the flags to the state of the lwgeom
 	*/
@@ -1017,7 +1213,7 @@ static LWPOINT* lwpoint_from_gserialized_buffer(uint8_t *data_ptr, uint8_t g_fla
 	point->flags = g_flags;
 
 	data_ptr += 4; /* Skip past the type. */
-	npoints = lw_get_uint32_t(data_ptr); /* Zero => empty geometry */
+	npoints = gserialized_get_uint32_t(data_ptr); /* Zero => empty geometry */
 	data_ptr += 4; /* Skip past the npoints. */
 
 	if ( npoints > 0 )
@@ -1048,12 +1244,12 @@ static LWLINE* lwline_from_gserialized_buffer(uint8_t *data_ptr, uint8_t g_flags
 	line->flags = g_flags;
 
 	data_ptr += 4; /* Skip past the type. */
-	npoints = lw_get_uint32_t(data_ptr); /* Zero => empty geometry */
+	npoints = gserialized_get_uint32_t(data_ptr); /* Zero => empty geometry */
 	data_ptr += 4; /* Skip past the npoints. */
 
 	if ( npoints > 0 )
 		line->points = ptarray_construct_reference_data(FLAGS_GET_Z(g_flags), FLAGS_GET_M(g_flags), npoints, data_ptr);
-		
+
 	else
 		line->points = ptarray_construct(FLAGS_GET_Z(g_flags), FLAGS_GET_M(g_flags), 0); /* Empty linestring */
 
@@ -1082,7 +1278,7 @@ static LWPOLY* lwpoly_from_gserialized_buffer(uint8_t *data_ptr, uint8_t g_flags
 	poly->flags = g_flags;
 
 	data_ptr += 4; /* Skip past the polygontype. */
-	nrings = lw_get_uint32_t(data_ptr); /* Zero => empty geometry */
+	nrings = gserialized_get_uint32_t(data_ptr); /* Zero => empty geometry */
 	poly->nrings = nrings;
 	LWDEBUGF(4, "nrings = %d", nrings);
 	data_ptr += 4; /* Skip past the nrings. */
@@ -1105,12 +1301,12 @@ static LWPOLY* lwpoly_from_gserialized_buffer(uint8_t *data_ptr, uint8_t g_flags
 		uint32_t npoints = 0;
 
 		/* Read in the number of points. */
-		npoints = lw_get_uint32_t(data_ptr);
+		npoints = gserialized_get_uint32_t(data_ptr);
 		data_ptr += 4;
 
 		/* Make a point array for the ring, and move the ordinate pointer past the ring ordinates. */
 		poly->rings[i] = ptarray_construct_reference_data(FLAGS_GET_Z(g_flags), FLAGS_GET_M(g_flags), npoints, ordinate_ptr);
-		
+
 		ordinate_ptr += sizeof(double) * FLAGS_NDIMS(g_flags) * npoints;
 	}
 
@@ -1135,11 +1331,11 @@ static LWTRIANGLE* lwtriangle_from_gserialized_buffer(uint8_t *data_ptr, uint8_t
 	triangle->flags = g_flags;
 
 	data_ptr += 4; /* Skip past the type. */
-	npoints = lw_get_uint32_t(data_ptr); /* Zero => empty geometry */
+	npoints = gserialized_get_uint32_t(data_ptr); /* Zero => empty geometry */
 	data_ptr += 4; /* Skip past the npoints. */
 
 	if ( npoints > 0 )
-		triangle->points = ptarray_construct_reference_data(FLAGS_GET_Z(g_flags), FLAGS_GET_M(g_flags), npoints, data_ptr);		
+		triangle->points = ptarray_construct_reference_data(FLAGS_GET_Z(g_flags), FLAGS_GET_M(g_flags), npoints, data_ptr);
 	else
 		triangle->points = ptarray_construct(FLAGS_GET_Z(g_flags), FLAGS_GET_M(g_flags), 0); /* Empty triangle */
 
@@ -1166,11 +1362,11 @@ static LWCIRCSTRING* lwcircstring_from_gserialized_buffer(uint8_t *data_ptr, uin
 	circstring->flags = g_flags;
 
 	data_ptr += 4; /* Skip past the circstringtype. */
-	npoints = lw_get_uint32_t(data_ptr); /* Zero => empty geometry */
+	npoints = gserialized_get_uint32_t(data_ptr); /* Zero => empty geometry */
 	data_ptr += 4; /* Skip past the npoints. */
 
 	if ( npoints > 0 )
-		circstring->points = ptarray_construct_reference_data(FLAGS_GET_Z(g_flags), FLAGS_GET_M(g_flags), npoints, data_ptr);		
+		circstring->points = ptarray_construct_reference_data(FLAGS_GET_Z(g_flags), FLAGS_GET_M(g_flags), npoints, data_ptr);
 	else
 		circstring->points = ptarray_construct(FLAGS_GET_Z(g_flags), FLAGS_GET_M(g_flags), 0); /* Empty circularstring */
 
@@ -1192,7 +1388,7 @@ static LWCOLLECTION* lwcollection_from_gserialized_buffer(uint8_t *data_ptr, uin
 
 	assert(data_ptr);
 
-	type = lw_get_uint32_t(data_ptr);
+	type = gserialized_get_uint32_t(data_ptr);
 	data_ptr += 4; /* Skip past the type. */
 
 	collection = (LWCOLLECTION*)lwalloc(sizeof(LWCOLLECTION));
@@ -1201,7 +1397,7 @@ static LWCOLLECTION* lwcollection_from_gserialized_buffer(uint8_t *data_ptr, uin
 	collection->type = type;
 	collection->flags = g_flags;
 
-	ngeoms = lw_get_uint32_t(data_ptr);
+	ngeoms = gserialized_get_uint32_t(data_ptr);
 	collection->ngeoms = ngeoms; /* Zero => empty geometry */
 	data_ptr += 4; /* Skip past the ngeoms. */
 
@@ -1215,7 +1411,7 @@ static LWCOLLECTION* lwcollection_from_gserialized_buffer(uint8_t *data_ptr, uin
 
 	for ( i = 0; i < ngeoms; i++ )
 	{
-		uint32_t subtype = lw_get_uint32_t(data_ptr);
+		uint32_t subtype = gserialized_get_uint32_t(data_ptr);
 		size_t subsize = 0;
 
 		if ( ! lwcollection_allows_subtype(type, subtype) )
@@ -1240,7 +1436,7 @@ LWGEOM* lwgeom_from_gserialized_buffer(uint8_t *data_ptr, uint8_t g_flags, size_
 
 	assert(data_ptr);
 
-	type = lw_get_uint32_t(data_ptr);
+	type = gserialized_get_uint32_t(data_ptr);
 
 	LWDEBUGF(2, "Got type %d (%s), hasz=%d hasm=%d geodetic=%d hasbox=%d", type, lwtype_name(type),
 		FLAGS_GET_Z(g_flags), FLAGS_GET_M(g_flags), FLAGS_GET_GEODETIC(g_flags), FLAGS_GET_BBOX(g_flags));
