@@ -56,7 +56,7 @@ double longitude_radians_normalize(double lon)
 
 	if ( lon < -1.0 * M_PI )
 		lon = 2.0 * M_PI + lon;
-		
+
 	if ( lon == -2.0 * M_PI )
 		lon *= -1.0;
 
@@ -183,11 +183,11 @@ gbox_angular_height(const GBOX* gbox)
 	double zmin = FLT_MAX;
 	double zmax = -1 * FLT_MAX;
 	POINT3D pt;
-	
+
 	/* Take a copy of the box corners so we can treat them as a list */
 	/* Elements are xmin, xmax, ymin, ymax, zmin, zmax */
 	memcpy(d, &(gbox->xmin), 6*sizeof(double));
-	
+
 	/* Generate all 8 corner vectors of the box */
 	for ( i = 0; i < 8; i++ )
 	{
@@ -231,7 +231,7 @@ gbox_angular_width(const GBOX* gbox)
 		{
 			double angle, dotprod;
 			POINT3D pt_n;
-		
+
 			pt_n.x = d[i / 2];
 			pt_n.y = d[2 + (i % 2)];
 			magnitude = sqrt(pt_n.x*pt_n.x + pt_n.y*pt_n.y);
@@ -248,7 +248,7 @@ gbox_angular_width(const GBOX* gbox)
 			}
 		}
 	}
-	
+
 	/* Return the distance between the two furthest vectors */
 	return maxangle;
 }
@@ -265,24 +265,24 @@ gbox_centroid(const GBOX* gbox, POINT2D* out)
 	/* Take a copy of the box corners so we can treat them as a list */
 	/* Elements are xmin, xmax, ymin, ymax, zmin, zmax */
 	memcpy(d, &(gbox->xmin), 6*sizeof(double));
-	
+
 	/* Zero out our return vector */
 	pt.x = pt.y = pt.z = 0.0;
 
 	for ( i = 0; i < 8; i++ )
 	{
 		POINT3D pt_n;
-	
+
 		pt_n.x = d[i / 4];
 		pt_n.y = d[2 + ((i % 4) / 2)];
 		pt_n.z = d[4 + (i % 2)];
 		normalize(&pt_n);
-	
+
 		pt.x += pt_n.x;
 		pt.y += pt_n.y;
-		pt.z += pt_n.z;		
+		pt.z += pt_n.z;
 	}
-	
+
 	pt.x /= 8.0;
 	pt.y /= 8.0;
 	pt.z /= 8.0;
@@ -291,7 +291,7 @@ gbox_centroid(const GBOX* gbox, POINT2D* out)
 	cart2geog(&pt, &g);
 	out->x = longitude_degrees_normalize(rad2deg(g.lon));
 	out->y = latitude_degrees_normalize(rad2deg(g.lat));
-	
+
 	return LW_SUCCESS;
 }
 
@@ -475,10 +475,10 @@ double vector_angle(const POINT3D* v1, const POINT3D* v2)
 	cross_product(v1, v2, &normal);
 	normalize(&normal);
 	cross_product(&normal, v1, &v3);
-	
+
 	x = dot_product(v1, v2);
 	y = dot_product(v2, &v3);
-	
+
 	angle = atan2(y, x);
 	return angle;
 }
@@ -507,7 +507,7 @@ void unit_normal(const POINT3D *P1, const POINT3D *P2, POINT3D *normal)
 {
 	double p_dot = dot_product(P1, P2);
 	POINT3D P3;
-	
+
 	/* If edge is really large, calculate a narrower equivalent angle A1/A3. */
 	if ( p_dot < 0 )
 	{
@@ -525,7 +525,7 @@ void unit_normal(const POINT3D *P1, const POINT3D *P2, POINT3D *normal)
 	{
 		P3 = *P2;
 	}
-	
+
 	/* Normals to the A-plane and B-plane */
 	cross_product(P1, &P3, normal);
 	normalize(normal);
@@ -543,26 +543,26 @@ void vector_rotate(const POINT3D* v1, const POINT3D* v2, double angle, POINT3D*
 	double uxuy, uyuz, uxuz;
 	double ux2, uy2, uz2;
 	double rxx, rxy, rxz, ryx, ryy, ryz, rzx, rzy, rzz;
-	
+
 	/* Need a unit vector normal to rotate around */
 	unit_normal(v1, v2, &u);
-	
+
 	uxuy = u.x * u.y;
 	uxuz = u.x * u.z;
 	uyuz = u.y * u.z;
-	
+
 	ux2 = u.x * u.x;
 	uy2 = u.y * u.y;
 	uz2 = u.z * u.z;
-	
+
 	rxx = cos_a + ux2 * (1 - cos_a);
 	rxy = uxuy * (1 - cos_a) - u.z * sin_a;
 	rxz = uxuz * (1 - cos_a) + u.y * sin_a;
-	
+
 	ryx = uxuy * (1 - cos_a) + u.z * sin_a;
 	ryy = cos_a + uy2 * (1 - cos_a);
 	ryz = uyuz * (1 - cos_a) - u.x * sin_a;
-	
+
 	rzx = uxuz * (1 - cos_a) - u.y * sin_a;
 	rzy = uyuz * (1 - cos_a) + u.x * sin_a;
 	rzz = cos_a + uz2 * (1 - cos_a);
@@ -672,7 +672,7 @@ edge_point_side(const GEOGRAPHIC_EDGE *e, const GEOGRAPHIC_POINT *p)
 		LWDEBUG(4, "point is on plane (dot product is zero)");
 		return 0;
 	}
-	
+
 	if ( w < 0 )
 		return -1;
 	else
@@ -690,7 +690,7 @@ sphere_angle(const GEOGRAPHIC_POINT *a, const GEOGRAPHIC_POINT *b,  const GEOGRA
 	robust_cross_product(b, c, &normal2);
 	normalize(&normal1);
 	normalize(&normal2);
-	return sphere_distance_cartesian(&normal1, &normal2);	
+	return sphere_distance_cartesian(&normal1, &normal2);
 }
 
 /**
@@ -709,18 +709,18 @@ sphere_signed_area(const GEOGRAPHIC_POINT *a, const GEOGRAPHIC_POINT *b, const G
 	double area_radians = 0.0;
 	int side;
 	GEOGRAPHIC_EDGE e;
-	
+
 	angle_a = sphere_angle(b,a,c);
 	angle_b = sphere_angle(a,b,c);
 	angle_c = sphere_angle(b,c,a);
-	
+
 	area_radians = angle_a + angle_b + angle_c - M_PI;
 
 	/* What's the direction of the B/C edge? */
 	e.start = *a;
 	e.end = *b;
 	side = edge_point_side(&e, c);
-	
+
 	/* Co-linear points implies no area */
 	if ( side == 0 )
 		return 0.0;
@@ -742,7 +742,7 @@ int edge_point_on_plane(const GEOGRAPHIC_EDGE *e, const GEOGRAPHIC_POINT *p)
 	int side = edge_point_side(e, p);
 	if ( side == 0 )
 		return LW_TRUE;
-		
+
 	return LW_FALSE;
 }
 
@@ -941,7 +941,7 @@ double sphere_direction(const GEOGRAPHIC_POINT *s, const GEOGRAPHIC_POINT *e, do
 {
 	double heading = 0.0;
 	double f;
-	
+
 	/* Starting from the poles? Special case. */
 	if ( FP_IS_ZERO(cos(s->lat)) )
 		return (s->lat > 0.0) ? M_PI : 0.0;
@@ -1295,7 +1295,7 @@ int sphere_project(const GEOGRAPHIC_POINT *r, double distance, double azimuth, G
 	{
 		lon2 = lon1 + atan2(sin(azimuth)*sin(d)*cos(lat1), cos(d)-sin(lat1)*sin(lat2));
 	}
-	
+
 	if ( isnan(lat2) || isnan(lon2) )
 		return LW_FAILURE;
 
@@ -1380,18 +1380,18 @@ int edge_calculate_gbox(const POINT3D *A1, const POINT3D *A2, GBOX *gbox)
 	/* Initialize the box with the edge end points */
 	gbox_init_point3d(A1, gbox);
 	gbox_merge_point3d(A2, gbox);
-	
+
 	/* Zero length edge, just return! */
 	if ( p3d_same(A1, A2) )
 		return LW_SUCCESS;
-	
+
 	/* Error out on antipodal edge */
 	if ( FP_EQUALS(A1->x, -1*A2->x) && FP_EQUALS(A1->y, -1*A2->y) && FP_EQUALS(A1->z, -1*A2->z) )
 	{
 		lwerror("Antipodal (180 degrees long) edge detected!");
 		return LW_FAILURE;
 	}
-	
+
 	/* Create A3, a vector in the plane of A1/A2, orthogonal to A1  */
 	unit_normal(A1, A2, &AN);
 	unit_normal(&AN, A1, &A3);
@@ -1406,12 +1406,12 @@ int edge_calculate_gbox(const POINT3D *A1, const POINT3D *A2, GBOX *gbox)
 	memset(X, 0, sizeof(POINT3D) * 6);
 	X[0].x = X[2].y = X[4].z =  1.0;
 	X[1].x = X[3].y = X[5].z = -1.0;
-	
+
 	/* Initialize a 2-space origin point. */
 	O.x = O.y = 0.0;
 	/* What side of the line joining R1/R2 is O? */
 	o_side = lw_segment_side(&R1, &R2, &O);
-	
+
 	/* Add any extrema! */
 	for ( i = 0; i < 6; i++ )
 	{
@@ -1419,7 +1419,7 @@ int edge_calculate_gbox(const POINT3D *A1, const POINT3D *A2, GBOX *gbox)
 		RX.x = dot_product(&(X[i]), A1);
 		RX.y = dot_product(&(X[i]), &A3);
 		normalize2d(&RX);
-		
+
 		/* Any axis end on the side of R1/R2 opposite the origin */
 		/* is an extreme point in the arc, so we add the 3-space */
 		/* version of the point on R1/R2 to the gbox */
@@ -1429,7 +1429,7 @@ int edge_calculate_gbox(const POINT3D *A1, const POINT3D *A2, GBOX *gbox)
 			Xn.x = RX.x * A1->x + RX.y * A3.x;
 			Xn.y = RX.x * A1->y + RX.y * A3.y;
 			Xn.z = RX.x * A1->z + RX.y * A3.z;
-			
+
 			gbox_merge_point3d(&Xn, gbox);
 		}
 	}
@@ -1438,7 +1438,7 @@ int edge_calculate_gbox(const POINT3D *A1, const POINT3D *A2, GBOX *gbox)
 }
 
 void lwpoly_pt_outside(const LWPOLY *poly, POINT2D *pt_outside)
-{	
+{
 	/* Make sure we have boxes */
 	if ( poly->bbox )
 	{
@@ -1554,11 +1554,11 @@ ptarray_segmentize_sphere(const POINTARRAY *pa_in, double max_seg_length)
 	POINT4D p1, p2, p;
 	GEOGRAPHIC_POINT g1, g2, g;
 	double d;
-	
+
 	/* Just crap out on crazy input */
 	if ( ! pa_in )
 		lwerror("ptarray_segmentize_sphere: null input pointarray");
-	if ( max_seg_length <= 0.0 )	
+	if ( max_seg_length <= 0.0 )
 		lwerror("ptarray_segmentize_sphere: maximum segment length must be positive");
 
 	/* Empty starting array */
@@ -1569,12 +1569,12 @@ ptarray_segmentize_sphere(const POINTARRAY *pa_in, double max_seg_length)
 	ptarray_append_point(pa_out, &p1, LW_FALSE);
 	geographic_point_init(p1.x, p1.y, &g1);
 	pa_in_offset++;
-	
+
 	while ( pa_in_offset < pa_in->npoints )
 	{
 		getPoint4d_p(pa_in, pa_in_offset, &p2);
 		geographic_point_init(p2.x, p2.y, &g2);
-		
+
 		/* Skip duplicate points (except in case of 2-point lines!) */
 		if ( (pa_in->npoints > 2) && p4d_same(&p1, &p2) )
 		{
@@ -1587,7 +1587,7 @@ ptarray_segmentize_sphere(const POINTARRAY *pa_in, double max_seg_length)
 
 		/* How long is this edge? */
 		d = sphere_distance(&g1, &g2);
-		
+
 		/* We need to segmentize this edge */
 		if ( d > max_seg_length )
 		{
@@ -1599,7 +1599,7 @@ ptarray_segmentize_sphere(const POINTARRAY *pa_in, double max_seg_length)
 			/* The independent Z/M values on the ptarray */
 			if ( hasz ) dzz = (p2.z - p1.z) / nsegs;
 			if ( hasm ) dmm = (p2.m - p1.m) / nsegs;
-			
+
 			g = g1;
 			p = p1;
 			for ( i = 0; i < nsegs - 1; i++ )
@@ -1621,7 +1621,7 @@ ptarray_segmentize_sphere(const POINTARRAY *pa_in, double max_seg_length)
 					p.m += dmm;
 				ptarray_append_point(pa_out, &p, LW_FALSE);
 			}
-			
+
 			ptarray_append_point(pa_out, &p2, LW_FALSE);
 		}
 		/* This edge is already short enough */
@@ -1635,8 +1635,8 @@ ptarray_segmentize_sphere(const POINTARRAY *pa_in, double max_seg_length)
 		g1 = g2;
 		pa_in_offset++;
 	}
-	
-	return pa_out;	
+
+	return pa_out;
 }
 
 /**
@@ -1653,15 +1653,15 @@ lwgeom_segmentize_sphere(const LWGEOM *lwg_in, double max_seg_length)
 	LWPOLY *lwpoly_in, *lwpoly_out;
 	LWCOLLECTION *lwcol_in, *lwcol_out;
 	int i;
-	
+
 	/* Reflect NULL */
 	if ( ! lwg_in )
 		return NULL;
-		
+
 	/* Clone empty */
 	if ( lwgeom_is_empty(lwg_in) )
 		return lwgeom_clone(lwg_in);
-	
+
 	switch (lwg_in->type)
 	{
 	case MULTIPOINTTYPE:
@@ -1699,7 +1699,7 @@ lwgeom_segmentize_sphere(const LWGEOM *lwg_in, double max_seg_length)
 		        lwg_in->type, lwtype_name(lwg_in->type));
 		break;
 	}
-	
+
 	lwerror("lwgeom_segmentize_sphere got to the end of the function, should not happen");
 	return NULL;
 }
@@ -1716,16 +1716,16 @@ ptarray_area_sphere(const POINTARRAY *pa)
 	const POINT2D *p;
 	GEOGRAPHIC_POINT a, b, c;
 	double area = 0.0;
-	
+
 	/* Return zero on nonsensical inputs */
 	if ( ! pa || pa->npoints < 4 )
 		return 0.0;
-	
+
 	p = getPoint2d_cp(pa, 0);
 	geographic_point_init(p->x, p->y, &a);
 	p = getPoint2d_cp(pa, 1);
 	geographic_point_init(p->x, p->y, &b);
-	
+
 	for ( i = 2; i < pa->npoints-1; i++ )
 	{
 		p = getPoint2d_cp(pa, i);
@@ -1733,7 +1733,7 @@ ptarray_area_sphere(const POINTARRAY *pa)
 		area += sphere_signed_area(&a, &b, &c);
 		b = c;
 	}
-	
+
 	return fabs(area);
 }
 
@@ -2010,25 +2010,27 @@ LWPOINT* lwgeom_project_spheroid(const LWPOINT *r, const SPHEROID *spheroid, dou
 	POINTARRAY *pa;
 	LWPOINT *lwp;
 
-	/* Check the azimuth validity, convert to radians */
-	if ( azimuth < -2.0 * M_PI || azimuth > 2.0 * M_PI )
+	/* Normalize distance to be positive*/
+	if ( distance < 0.0 ) {
+		distance = -distance;
+		azimuth += M_PI;
+	}
+
+	/* Normalize azimuth */
+	azimuth -= 2.0 * M_PI * floor(azimuth / (2.0 * M_PI));
+
+	/* Check the distance validity */
+	if ( distance > (M_PI * spheroid->radius) )
 	{
-		lwerror("Azimuth must be between -2PI and 2PI");
+		lwerror("Distance must not be greater than %g", M_PI * spheroid->radius);
 		return NULL;
 	}
 
-	/* Check the distance validity */
-	if ( distance < 0.0 || distance > (M_PI * spheroid->radius) )
-	{
-		lwerror("Distance must be between 0 and %g", M_PI * spheroid->radius);
-		return NULL;
-	}
-		
 	/* Convert to ta geodetic point */
 	x = lwpoint_get_x(r);
 	y = lwpoint_get_y(r);
 	geographic_point_init(x, y, &geo_source);
-	
+
 	/* Try the projection */
 	if( spheroid_project(&geo_source, spheroid, distance, azimuth, &geo_dest) == LW_FAILURE )
 	{
@@ -2036,7 +2038,7 @@ LWPOINT* lwgeom_project_spheroid(const LWPOINT *r, const SPHEROID *spheroid, dou
 		lwerror("Unable to project from (%g %g) with azimuth %g and distance %g", x, y, azimuth, distance);
 		return NULL;
 	}
-	
+
 	/* Build the output LWPOINT */
 	pa = ptarray_construct(0, 0, 1);
 	pt_dest.x = rad2deg(longitude_radians_normalize(geo_dest.lon));
@@ -2071,13 +2073,13 @@ double lwgeom_azumith_spheroid(const LWPOINT *r, const LWPOINT *s, const SPHEROI
 	x2 = lwpoint_get_x(s);
 	y2 = lwpoint_get_y(s);
 	geographic_point_init(x2, y2, &g2);
-	
+
 	/* Same point, return NaN */
 	if ( FP_EQUALS(x1, x2) && FP_EQUALS(y1, y2) )
 	{
 		return NAN;
 	}
-	
+
 	/* Do the direction calculation */
 	return spheroid_direction(&g1, &g2, spheroid);
 }
@@ -2096,10 +2098,10 @@ double lwgeom_distance_spheroid(const LWGEOM *lwgeom1, const LWGEOM *lwgeom2, co
 
 	gbox_init(&gbox1);
 	gbox_init(&gbox2);
-	
+
 	assert(lwgeom1);
 	assert(lwgeom2);
-	
+
 	LWDEBUGF(4, "entered function, tolerance %.8g", tolerance);
 
 	/* What's the distance to an empty geometry? We don't know.
@@ -2174,7 +2176,7 @@ double lwgeom_distance_spheroid(const LWGEOM *lwgeom1, const LWGEOM *lwgeom2, co
 		{
 			return 0.0;
 		}
-		
+
 		/* Not inside, so what's the actual distance? */
 		for ( i = 0; i < lwpoly->nrings; i++ )
 		{
@@ -2315,18 +2317,19 @@ int lwgeom_covers_lwgeom_sphere(const LWGEOM *lwgeom1, const LWGEOM *lwgeom2)
 	int type1, type2;
 	GBOX gbox1, gbox2;
 	gbox1.flags = gbox2.flags = 0;
-		
+
 	assert(lwgeom1);
 	assert(lwgeom2);
 
 	type1 = lwgeom1->type;
 	type2 = lwgeom2->type;
 
-	/* Currently a restricted implementation */
-	if ( ! ( (type1 == POLYGONTYPE || type1 == MULTIPOLYGONTYPE || type1 == COLLECTIONTYPE) &&
-	         (type2 == POINTTYPE || type2 == MULTIPOINTTYPE || type2 == COLLECTIONTYPE) ) )
+	/* dim(geom2) > dim(geom1) always returns false (because geom2 is bigger) */
+	if ( (type1 == POINTTYPE && type2 == LINETYPE)
+		|| (type1 == POINTTYPE && type2 == POLYGONTYPE)
+		|| (type1 == LINETYPE && type2 == POLYGONTYPE) )
 	{
-		lwerror("lwgeom_covers_lwgeom_sphere: only POLYGON covers POINT tests are currently supported");
+		LWDEBUG(4, "dimension of geom2 is bigger than geom1");
 		return LW_FALSE;
 	}
 
@@ -2350,6 +2353,26 @@ int lwgeom_covers_lwgeom_sphere(const LWGEOM *lwgeom1, const LWGEOM *lwgeom2)
 		getPoint2d_p(((LWPOINT*)lwgeom2)->point, 0, &pt_to_test);
 		return lwpoly_covers_point2d((LWPOLY*)lwgeom1, &pt_to_test);
 	}
+	else if ( type1 == POLYGONTYPE && type2 == LINETYPE)
+	{
+		return lwpoly_covers_lwline((LWPOLY*)lwgeom1, (LWLINE*)lwgeom2);
+	}
+	else if ( type1 == POLYGONTYPE && type2 == POLYGONTYPE)
+	{
+		return lwpoly_covers_lwpoly((LWPOLY*)lwgeom1, (LWPOLY*)lwgeom2);
+	}
+	else if ( type1 == LINETYPE && type2 == POINTTYPE)
+	{
+		return lwline_covers_lwpoint((LWLINE*)lwgeom1, (LWPOINT*)lwgeom2);
+	}
+	else if ( type1 == LINETYPE && type2 == LINETYPE)
+	{
+		return lwline_covers_lwline((LWLINE*)lwgeom1, (LWLINE*)lwgeom2);
+	}
+	else if ( type1 == POINTTYPE && type2 == POINTTYPE)
+	{
+		return lwpoint_same((LWPOINT*)lwgeom1, (LWPOINT*)lwgeom2);
+	}
 
 	/* If any of the first argument parts covers the second argument, it's true */
 	if ( lwtype_is_collection( type1 ) )
@@ -2464,6 +2487,279 @@ int lwpoly_covers_point2d(const LWPOLY *poly, const POINT2D *pt_to_test)
 	return LW_TRUE;
 }
 
+/**
+ * Given a polygon1 check if all points of polygon2 are inside polygon1 and no
+ * intersections of the polygon edges occur.
+ * return LW_TRUE if polygon is inside or on edge of polygon.
+ */
+int lwpoly_covers_lwpoly(const LWPOLY *poly1, const LWPOLY *poly2)
+{
+	int i;
+
+	/* Nulls and empties don't contain anything! */
+	if ( ! poly1 || lwgeom_is_empty((LWGEOM*)poly1) )
+	{
+		LWDEBUG(4,"returning false, geometry1 is empty or null");
+		return LW_FALSE;
+	}
+
+	/* Nulls and empties don't contain anything! */
+	if ( ! poly2 || lwgeom_is_empty((LWGEOM*)poly2) )
+	{
+		LWDEBUG(4,"returning false, geometry2 is empty or null");
+		return LW_FALSE;
+	}
+
+	/* check if all vertices of poly2 are inside poly1 */
+	for (i = 0; i < poly2->nrings; i++)
+	{
+
+		/* every other ring is a hole, check if point is inside the actual polygon */
+		if ( i % 2 == 0)
+		{
+			if (LW_FALSE == lwpoly_covers_pointarray(poly1, poly2->rings[i]))
+			{
+				LWDEBUG(4,"returning false, geometry2 has point outside of geometry1");
+				return LW_FALSE;
+			}
+		}
+		else
+		{
+			if (LW_TRUE == lwpoly_covers_pointarray(poly1, poly2->rings[i]))
+			{
+				LWDEBUG(4,"returning false, geometry2 has point inside a hole of geometry1");
+				return LW_FALSE;
+			}
+		}
+	}
+
+	/* check for any edge intersections, so nothing is partially outside of poly1 */
+	for (i = 0; i < poly2->nrings; i++)
+	{
+		if (LW_TRUE == lwpoly_intersects_line(poly1, poly2->rings[i]))
+		{
+			LWDEBUG(4,"returning false, geometry2 is partially outside of geometry1");
+			return LW_FALSE;
+		}
+	}
+
+	/* no abort condition found, so the poly2 should be completly inside poly1 */
+	return LW_TRUE;
+}
+
+/**
+ *
+ */
+int lwpoly_covers_lwline(const LWPOLY *poly, const LWLINE *line)
+{
+   /* Nulls and empties don't contain anything! */
+   if ( ! poly || lwgeom_is_empty((LWGEOM*)poly) )
+   {
+	   LWDEBUG(4,"returning false, geometry1 is empty or null");
+	   return LW_FALSE;
+   }
+
+   /* Nulls and empties don't contain anything! */
+   if ( ! line || lwgeom_is_empty((LWGEOM*)line) )
+   {
+	   LWDEBUG(4,"returning false, geometry2 is empty or null");
+	   return LW_FALSE;
+   }
+
+   if (LW_FALSE == lwpoly_covers_pointarray(poly, line->points))
+   {
+	   LWDEBUG(4,"returning false, geometry2 has point outside of geometry1");
+	   return LW_FALSE;
+   }
+
+   /* check for any edge intersections, so nothing is partially outside of poly1 */
+   if (LW_TRUE == lwpoly_intersects_line(poly, line->points))
+   {
+	   LWDEBUG(4,"returning false, geometry2 is partially outside of geometry1");
+	   return LW_FALSE;
+   }
+
+   /* no abort condition found, so the poly2 should be completly inside poly1 */
+   return LW_TRUE;
+}
+
+/**
+ * return LW_TRUE if all points are inside the polygon
+ */
+int lwpoly_covers_pointarray(const LWPOLY* lwpoly, const POINTARRAY* pta)
+{
+	int i;
+	for (i = 0; i < pta->npoints; i++) {
+		const POINT2D* pt_to_test = getPoint2d_cp(pta, i);
+
+		if ( LW_FALSE == lwpoly_covers_point2d(lwpoly, pt_to_test) ) {
+			LWDEBUG(4,"returning false, geometry2 has point outside of geometry1");
+			return LW_FALSE;
+		}
+	}
+
+	return LW_TRUE;
+}
+
+/**
+ * Checks if any edges of lwpoly intersect with the line formed by the pointarray
+ * return LW_TRUE if any intersection beetween the given polygon and the line
+ */
+int lwpoly_intersects_line(const LWPOLY* lwpoly, const POINTARRAY* line)
+{
+	int i, j, k;
+	POINT3D pa1, pa2, pb1, pb2;
+	for (i = 0; i < lwpoly->nrings; i++)
+	{
+		for (j = 0; j < lwpoly->rings[i]->npoints - 1; j++)
+		{
+			const POINT2D* a1 = getPoint2d_cp(lwpoly->rings[i], j);
+			const POINT2D* a2 = getPoint2d_cp(lwpoly->rings[i], j+1);
+
+			/* Set up our stab line */
+			ll2cart(a1, &pa1);
+			ll2cart(a2, &pa2);
+
+			for (k = 0; k < line->npoints - 1; k++)
+			{
+				const POINT2D* b1 = getPoint2d_cp(line, k);
+				const POINT2D* b2 = getPoint2d_cp(line, k+1);
+
+				/* Set up our stab line */
+				ll2cart(b1, &pb1);
+				ll2cart(b2, &pb2);
+
+				int inter = edge_intersects(&pa1, &pa2, &pb1, &pb2);
+
+				/* ignore same edges */
+				if (inter & PIR_INTERSECTS
+					&& !(inter & PIR_B_TOUCH_RIGHT || inter & PIR_COLINEAR) )
+				{
+					return LW_TRUE;
+				}
+			}
+		}
+	}
+
+	return LW_FALSE;
+}
+
+/**
+ * return LW_TRUE if any of the line segments covers the point
+ */
+int lwline_covers_lwpoint(const LWLINE* lwline, const LWPOINT* lwpoint)
+{
+	int i;
+	GEOGRAPHIC_POINT p;
+	GEOGRAPHIC_EDGE e;
+
+	for ( i = 0; i < lwline->points->npoints - 1; i++)
+	{
+		const POINT2D* a1 = getPoint2d_cp(lwline->points, i);
+		const POINT2D* a2 = getPoint2d_cp(lwline->points, i+1);
+
+		geographic_point_init(a1->x, a1->y, &(e.start));
+		geographic_point_init(a2->x, a2->y, &(e.end));
+
+		geographic_point_init(lwpoint_get_x(lwpoint), lwpoint_get_y(lwpoint), &p);
+
+		if ( edge_contains_point(&e, &p) ) {
+			return LW_TRUE;
+		}
+	}
+
+	return LW_FALSE;
+}
+
+/**
+ * Check if first and last point of line2 are covered by line1 and then each
+ * point in between has to be one line1 in the exact same order
+ * return LW_TRUE if all edge points of line2 are on line1
+ */
+int lwline_covers_lwline(const LWLINE* lwline1, const LWLINE* lwline2)
+{
+	int i, j;
+	GEOGRAPHIC_EDGE e1, e2;
+	GEOGRAPHIC_POINT p1, p2;
+	int start = LW_FALSE;
+	int changed = LW_FALSE;
+
+	/* first point on line */
+	if ( ! lwline_covers_lwpoint(lwline1, lwline_get_lwpoint(lwline2, 0)))
+	{
+		LWDEBUG(4,"returning false, first point of line2 is not covered by line1");
+		return LW_FALSE;
+	}
+
+	/* last point on line */
+	if ( ! lwline_covers_lwpoint(lwline1, lwline_get_lwpoint(lwline2, lwline2->points->npoints - 1)))
+	{
+		LWDEBUG(4,"returning false, last point of line2 is not covered by line1");
+		return LW_FALSE;
+	}
+
+	j = 0;
+	i = 0;
+	while (i < lwline1->points->npoints - 1 && j < lwline2->points->npoints - 1)
+	{
+		changed = LW_FALSE;
+		const POINT2D* a1 = getPoint2d_cp(lwline1->points, i);
+		const POINT2D* a2 = getPoint2d_cp(lwline1->points, i+1);
+		const POINT2D* b1 = getPoint2d_cp(lwline2->points, j);
+		const POINT2D* b2 = getPoint2d_cp(lwline2->points, j+1);
+
+		geographic_point_init(a1->x, a1->y, &(e1.start));
+		geographic_point_init(a2->x, a2->y, &(e1.end));
+		geographic_point_init(b1->x, b1->y, &p2);
+
+		/* we already know, that the last point is on line1, so we're done */
+		if ( j == lwline2->points->npoints - 1)
+		{
+			return LW_TRUE;
+		}
+		else if (start == LW_TRUE)
+		{
+			/* point is on current line1 edge, check next point in line2 */
+			if ( edge_contains_point(&e1, &p2)) {
+				j++;
+				changed = LW_TRUE;
+			}
+
+			geographic_point_init(a1->x, a1->y, &(e2.start));
+			geographic_point_init(a2->x, b2->y, &(e2.end));
+			geographic_point_init(a1->x, a1->y, &p1);
+
+			/* point is on current line2 edge, check next point in line1 */
+			if ( edge_contains_point(&e2, &p1)) {
+				i++;
+				changed = LW_TRUE;
+			}
+
+			/* no edge progressed -> point left one line */
+			if ( changed == LW_FALSE )
+			{
+				LWDEBUG(4,"returning false, found point not covered by both lines");
+				return LW_FALSE;
+			}
+			else
+			{
+				continue;
+			}
+		}
+
+		/* find first edge to cover line2 */
+		if (edge_contains_point(&e1, &p2))
+		{
+			start = LW_TRUE;
+		}
+
+		/* next line1 edge */
+		i++;
+	}
+
+	/* no uncovered point found */
+	return LW_TRUE;
+}
 
 /**
 * This function can only be used on LWGEOM that is built on top of
@@ -2511,13 +2807,13 @@ int ptarray_calculate_gbox_geodetic(const POINTARRAY *pa, GBOX *gbox)
 
 	p = getPoint2d_cp(pa, 0);
 	ll2cart(p, &A1);
-	
+
 	for ( i = 1; i < pa->npoints; i++ )
 	{
-		
+
 		p = getPoint2d_cp(pa, i);
 		ll2cart(p, &A2);
-		
+
 		edge_calculate_gbox(&A1, &A2, &edge_gbox);
 
 		/* Initialize the box */
@@ -2531,7 +2827,7 @@ int ptarray_calculate_gbox_geodetic(const POINTARRAY *pa, GBOX *gbox)
 		{
 			gbox_merge(&edge_gbox, gbox);
 		}
-		
+
 		A1 = A2;
 	}
 
@@ -2729,7 +3025,7 @@ int lwgeom_check_geodetic(const LWGEOM *geom)
 {
 	if ( lwgeom_is_empty(geom) )
 		return LW_TRUE;
-		
+
 	switch (geom->type)
 	{
 	case POINTTYPE:
@@ -2880,7 +3176,7 @@ double ptarray_length_spheroid(const POINTARRAY *pa, const SPHEROID *s)
 		/* Add in the vertical displacement if we're in 3D */
 		if ( hasz )
 			seglength = sqrt( (zb-za)*(zb-za) + seglength*seglength );
-			
+
 		/* Add this segment length to the total */
 		length += seglength;
 
@@ -3057,11 +3353,11 @@ point_in_cone(const POINT3D *A1, const POINT3D *A2, const POINT3D *P)
 {
 	POINT3D AC; /* Center point of A1/A2 */
 	double min_similarity, similarity;
-	
+
 	/* The normalized sum bisects the angle between start and end. */
 	vector_sum(A1, A2, &AC);
 	normalize(&AC);
-	
+
 	/* The projection of start onto the center defines the minimum similarity */
 	min_similarity = dot_product(A1, &AC);
 
@@ -3097,7 +3393,7 @@ dot_product_side(const POINT3D *p, const POINT3D *q)
 
 	if ( FP_IS_ZERO(dp) )
 		return 0;
-		
+
 	return dp < 0.0 ? -1 : 1;
 }
 
@@ -3112,11 +3408,11 @@ edge_intersects(const POINT3D *A1, const POINT3D *A2, const POINT3D *B1, const P
 	double ab_dot;
 	int a1_side, a2_side, b1_side, b2_side;
 	int rv = PIR_NO_INTERACT;
-	
+
 	/* Normals to the A-plane and B-plane */
 	unit_normal(A1, A2, &AN);
 	unit_normal(B1, B2, &BN);
-	
+
 	/* Are A-plane and B-plane basically the same? */
 	ab_dot = dot_product(&AN, &BN);
 	if ( FP_EQUALS(fabs(ab_dot), 1.0) )
@@ -3130,7 +3426,7 @@ edge_intersects(const POINT3D *A1, const POINT3D *A2, const POINT3D *B1, const P
 		}
 		return rv;
 	}
-	
+
 	/* What side of plane-A and plane-B do the end points */
 	/* of A and B fall? */
 	a1_side = dot_product_side(&BN, A1);
@@ -3169,7 +3465,7 @@ edge_intersects(const POINT3D *A1, const POINT3D *A2, const POINT3D *B1, const P
 		{
 			return PIR_INTERSECTS;
 		}
-		
+
 		return PIR_NO_INTERACT;
 	}
 
@@ -3199,7 +3495,7 @@ edge_intersects(const POINT3D *A1, const POINT3D *A2, const POINT3D *B1, const P
 		/* Touches at B2, B1 is on what side? */
 		rv |= (b1_side < 0 ? PIR_B_TOUCH_RIGHT : PIR_B_TOUCH_LEFT);
 	}
-	
+
 	return rv;
 }
 
@@ -3245,16 +3541,16 @@ int ptarray_contains_point_sphere(const POINTARRAY *pa, const POINT2D *pt_outsid
 		{
 			continue;
 		}
-		
+
 		/* Our test point is on an edge end! Point is "in ring" by our definition */
 		if ( point3d_equals(&S1, &E1) )
 		{
 			return LW_TRUE;
 		}
-		
+
 		/* Calculate relationship between stab line and edge */
 		inter = edge_intersects(&S1, &S2, &E1, &E2);
-		
+
 		/* We have some kind of interaction... */
 		if ( inter & PIR_INTERSECTS )
 		{
@@ -3264,7 +3560,7 @@ int ptarray_contains_point_sphere(const POINTARRAY *pa, const POINT2D *pt_outsid
 			{
 				return LW_TRUE;
 			}
-			
+
 			/* It's a touching interaction, disregard all the left-side ones. */
 			/* It's a co-linear intersection, ignore those. */
 			if ( inter & PIR_B_TOUCH_RIGHT || inter & PIR_COLINEAR )
@@ -3283,7 +3579,7 @@ int ptarray_contains_point_sphere(const POINTARRAY *pa, const POINT2D *pt_outsid
 		{
 			LWDEBUGF(4,"    edge (%d) did not cross", i);
 		}
-		
+
 		/* Increment to next edge */
 		E1 = E2;
 	}
