@@ -19,6 +19,7 @@
  **********************************************************************
  *
  * Copyright 2009 Paul Ramsey <pramsey@cleverelephant.ca>
+ * Copyright 2017 Darafei Praliaskouski <me@komzpa.net>
  *
  **********************************************************************/
 
@@ -182,7 +183,7 @@ static bool box2df_intersection(const BOX2DF *a, const BOX2DF *b, BOX2DF *n)
 
 	if( a == NULL || b == NULL || n == NULL )
 		return FALSE;
-		
+
 	n->xmax = Min(a->xmax, b->xmax);
 	n->ymax = Min(a->ymax, b->ymax);
 	n->xmin = Max(a->xmin, b->xmin);
@@ -203,7 +204,7 @@ static float box2df_size(const BOX2DF *a)
 
 	if ( a == NULL )
 		return (float)0.0;
-		
+
 	if ( (a->xmax <= a->xmin) || (a->ymax <= a->ymin) )
 	{
 		result =  (float) 0.0;
@@ -216,6 +217,14 @@ static float box2df_size(const BOX2DF *a)
 	return result;
 }
 
+static float box2df_edge(const BOX2DF *a)
+{
+	if ( a == NULL )
+		return (float)0.0;
+
+	return ((a->xmax) - (a->xmin)) + ((a->ymax) - (a->ymin));
+}
+
 static float box2df_union_size(const BOX2DF *a, const BOX2DF *b)
 {
 	float result;
@@ -227,7 +236,7 @@ static float box2df_union_size(const BOX2DF *a, const BOX2DF *b)
 		elog(ERROR, "box2df_union_size received two null arguments");
 		return 0.0;
 	}
-	
+
 	if ( a == NULL )
 		return box2df_size(b);
 
@@ -243,6 +252,32 @@ static float box2df_union_size(const BOX2DF *a, const BOX2DF *b)
 }
 
 
+static float box2df_union_edge(const BOX2DF *a, const BOX2DF *b)
+{
+	float result;
+
+	POSTGIS_DEBUG(5,"entered function");
+
+	if ( a == NULL && b == NULL )
+	{
+		elog(ERROR, "box2df_union_edge received two null arguments");
+		return 0.0;
+	}
+
+	if ( a == NULL )
+		return box2df_edge(b);
+
+	if ( b == NULL )
+		return box2df_edge(a);
+
+	result = (Max(a->xmax,b->xmax) - Min(a->xmin,b->xmin)) +
+ 	         (Max(a->ymax,b->ymax) - Min(a->ymin,b->ymin));
+
+	POSTGIS_DEBUGF(5, "union edge of %s and %s is %.8g", box2df_to_string(a), box2df_to_string(b), result);
+
+	return result;
+}
+
 /* Convert a double-based GBOX into a float-based BOX2DF,
    ensuring the float box is larger than the double box */
 static inline int box2df_from_gbox_p(GBOX *box, BOX2DF *a)
@@ -554,22 +589,20 @@ gserialized_datum_get_box2df_p(Datum gsdatum, BOX2DF *box2df)
 	GSERIALIZED *gpart;
 	uint8_t flags;
 	int result = LW_SUCCESS;
+	int gpart_is_slice = FALSE;
 
 	POSTGIS_DEBUG(4, "entered function");
 
 	/*
-	** The most info we need is the 8 bytes of serialized header plus the
-	** of floats necessary to hold the bounding box.
+	** Because geometry is declared as "storage = main" anything large
+	** enough to take serious advantage of PG_DETOAST_DATUM_SLICE will have
+	** already been compressed, which means the entire object will be
+	** fetched and decompressed before a slice is taken, thus removing
+	** any efficiencies gained from slicing. We need to move to
+	** "storage = external" and implement our own geometry compressor
+	** before we can take advantage of sliced retrieval.
 	*/
-	if (VARATT_IS_EXTENDED(gsdatum))
-	{
-		gpart = (GSERIALIZED*)PG_DETOAST_DATUM_SLICE(gsdatum, 0, 8 + sizeof(BOX2DF));
-	}
-	else
-	{
-		gpart = (GSERIALIZED*)PG_DETOAST_DATUM(gsdatum);
-	}
-
+	gpart = (GSERIALIZED*)PG_DETOAST_DATUM(gsdatum);
 	flags = gpart->flags;
 
 	POSTGIS_DEBUGF(4, "got flags %d", gpart->flags);
@@ -586,27 +619,21 @@ gserialized_datum_get_box2df_p(Datum gsdatum, BOX2DF *box2df)
 	{
 		/* No, we need to calculate it from the full object. */
 		GBOX gbox;
-		GSERIALIZED *g = (GSERIALIZED*)PG_DETOAST_DATUM(gsdatum);
-
 		gbox_init(&gbox);
 
-		if (gserialized_get_gbox_p(g, &gbox) == LW_FAILURE)
+		result = gserialized_get_gbox_p(gpart, &gbox);
+		if ( result == LW_SUCCESS )
 		{
-			POSTGIS_DEBUG(4, "could not calculate bbox, returning failure");
-			POSTGIS_FREE_IF_COPY_P(gpart, gsdatum);
-			POSTGIS_FREE_IF_COPY_P(g, gsdatum);
-			return LW_FAILURE;
+			result = box2df_from_gbox_p(&gbox, box2df);
+		}
+		else
+		{
+			POSTGIS_DEBUG(4, "could not calculate bbox");
 		}
-		POSTGIS_FREE_IF_COPY_P(g, gsdatum);
-		result = box2df_from_gbox_p(&gbox, box2df);
 	}
 
 	POSTGIS_FREE_IF_COPY_P(gpart, gsdatum);
-
-	if ( result == LW_SUCCESS )
-	{
-		POSTGIS_DEBUGF(4, "got box2df %s", box2df_to_string(box2df));
-	}
+	POSTGIS_DEBUGF(4, "result = %d, got box2df %s", result, result == LW_SUCCESS ? box2df_to_string(box2df) : "NONE");
 
 	return result;
 }
@@ -722,13 +749,13 @@ Datum gserialized_distance_centroid_2d(PG_FUNCTION_ARGS)
 	BOX2DF b1, b2;
 	Datum gs1 = PG_GETARG_DATUM(0);
 	Datum gs2 = PG_GETARG_DATUM(1);
-	
+
 	POSTGIS_DEBUG(3, "entered function");
 
 	/* Must be able to build box for each argument (ie, not empty geometry). */
 	if ( (gserialized_datum_get_box2df_p(gs1, &b1) == LW_SUCCESS) &&
 	     (gserialized_datum_get_box2df_p(gs2, &b2) == LW_SUCCESS) )
-	{	
+	{
 		double distance = box2df_distance_leaf_centroid(&b1, &b2);
 		POSTGIS_DEBUGF(3, "got boxes %s and %s", box2df_to_string(&b1), box2df_to_string(&b2));
 		PG_RETURN_FLOAT8(distance);
@@ -742,13 +769,13 @@ Datum gserialized_distance_box_2d(PG_FUNCTION_ARGS)
 	BOX2DF b1, b2;
 	Datum gs1 = PG_GETARG_DATUM(0);
 	Datum gs2 = PG_GETARG_DATUM(1);
-	
+
 	POSTGIS_DEBUG(3, "entered function");
 
 	/* Must be able to build box for each argument (ie, not empty geometry). */
 	if ( (gserialized_datum_get_box2df_p(gs1, &b1) == LW_SUCCESS) &&
 	     (gserialized_datum_get_box2df_p(gs2, &b2) == LW_SUCCESS) )
-	{	
+	{
 		double distance = box2df_distance(&b1, &b2);
 		POSTGIS_DEBUGF(3, "got boxes %s and %s", box2df_to_string(&b1), box2df_to_string(&b2));
 		PG_RETURN_FLOAT8(distance);
@@ -988,7 +1015,7 @@ static inline bool gserialized_gist_consistent_leaf_2d(BOX2DF *key, BOX2DF *quer
 	case RTOldContainedByStrategyNumber:
 		retval = (bool) box2df_contains(query, key);
 		break;
-		
+
 	/* To one side */
 	case RTAboveStrategyNumber:
 		retval = (bool) box2df_above(key, query);
@@ -1015,8 +1042,8 @@ static inline bool gserialized_gist_consistent_leaf_2d(BOX2DF *key, BOX2DF *quer
 		break;
 	case RTOverLeftStrategyNumber:
 		retval = (bool) box2df_overleft(key, query);
-		break;		
-		
+		break;
+
 	default:
 		retval = FALSE;
 	}
@@ -1036,7 +1063,7 @@ static inline bool gserialized_gist_consistent_internal_2d(BOX2DF *key, BOX2DF *
 
 	switch (strategy)
 	{
-		
+
 	/* Basic overlaps */
 	case RTOverlapStrategyNumber:
 		retval = (bool) box2df_overlaps(key, query);
@@ -1050,7 +1077,7 @@ static inline bool gserialized_gist_consistent_internal_2d(BOX2DF *key, BOX2DF *
 	case RTOldContainedByStrategyNumber:
 		retval = (bool) box2df_overlaps(key, query);
 		break;
-		
+
 	/* To one side */
 	case RTAboveStrategyNumber:
 		retval = (bool)(!box2df_overbelow(key, query));
@@ -1078,7 +1105,7 @@ static inline bool gserialized_gist_consistent_internal_2d(BOX2DF *key, BOX2DF *
 	case RTOverLeftStrategyNumber:
 		retval = (bool)(!box2df_right(key, query));
 		break;
-		
+
 	default:
 		retval = FALSE;
 	}
@@ -1191,12 +1218,12 @@ Datum gserialized_gist_distance_2d(PG_FUNCTION_ARGS)
 		POSTGIS_DEBUG(4, "[GIST] null query_gbox_index!");
 		PG_RETURN_FLOAT8(FLT_MAX);
 	}
-	
+
 	/* Get the entry box */
 	entry_box = (BOX2DF*)DatumGetPointer(entry->key);
-	
+
 #if POSTGIS_PGSQL_VERSION >= 95
-	
+
 	/* Box-style distance test */
 	if ( strategy == 14 ) /* operator <#> */
 	{
@@ -1237,15 +1264,42 @@ Datum gserialized_gist_distance_2d(PG_FUNCTION_ARGS)
 		/* Calculate distance for internal nodes */
 		distance = (double)box2df_distance_node_centroid(entry_box, &query_box);
 	}
-#endif	
-	
+#endif
+
 	PG_RETURN_FLOAT8(distance);
 }
 
+/*
+** Function to pack floats of different realms
+** This function serves to pack bit flags inside float type
+** Resulted value represent can be from four different "realms"
+** Every value from realm 3 is greater than any value from realms 2, 1 and 0.
+** Every value from realm 2 is less than every value from realm 3 and greater
+** than any value from realm 1 and 0, and so on. Values from the same realm
+** loose two bits of precision. This technique is possible due to floating
+** point numbers specification according to IEEE 754: exponent bits are highest
+** (excluding sign bits, but here penalty is always positive). If float a is
+** greater than float b, integer A with same bit representation as a is greater
+** than integer B with same bits as b.
+*/
+static float pack_float(const float value, const int realm)
+{
+  union {
+    float f;
+    struct { unsigned value:31, sign:1; } vbits;
+    struct { unsigned value:29, realm:2, sign:1; } rbits;
+  } a;
+
+  a.f = value;
+  a.rbits.value = a.vbits.value >> 2;
+  a.rbits.realm = realm;
+
+  return a.f;
+}
+
 /*
 ** GiST support function. Calculate the "penalty" cost of adding this entry into an existing entry.
 ** Calculate the change in volume of the old entry once the new entry is added.
-** TODO: Re-evaluate this in light of R*Tree penalty approaches.
 */
 PG_FUNCTION_INFO_V1(gserialized_gist_penalty_2d);
 Datum gserialized_gist_penalty_2d(PG_FUNCTION_ARGS)
@@ -1254,7 +1308,7 @@ Datum gserialized_gist_penalty_2d(PG_FUNCTION_ARGS)
 	GISTENTRY *newentry = (GISTENTRY*) PG_GETARG_POINTER(1);
 	float *result = (float*) PG_GETARG_POINTER(2);
 	BOX2DF *gbox_index_orig, *gbox_index_new;
-	float size_union, size_orig;
+	float size_union, size_orig, edge_union, edge_orig;
 
 	POSTGIS_DEBUG(4, "[GIST] 'penalty' function called");
 
@@ -1274,6 +1328,37 @@ Datum gserialized_gist_penalty_2d(PG_FUNCTION_ARGS)
 	size_orig = box2df_size(gbox_index_orig);
 	*result = size_union - size_orig;
 
+	/* REALM 0: No extension is required, volume is zero, return edge */
+ 	/* REALM 1: No extension is required, return nonzero area */
+ 	/* REALM 2: Area extension is zero, return nonzero edge extension */
+ 	/* REALM 3: Area extension is nonzero, return it */
+
+ 	if( *result == 0 )
+ 	{
+		if (size_orig > 0)
+		{
+			*result = pack_float(size_orig, 1); /* REALM 1 */
+		}
+		else
+		{
+			edge_union = box2df_union_edge(gbox_index_orig, gbox_index_new);
+			edge_orig = box2df_edge(gbox_index_orig);
+ 			*result = edge_union - edge_orig;
+ 			if( *result == 0 )
+	 		{
+	 			*result = pack_float(edge_orig, 0); /* REALM 0 */
+ 			}
+ 			else
+ 			{
+ 				*result = pack_float(*result, 2); /* REALM 2 */
+ 			}
+		}
+ 	}
+ 	else
+ 	{
+ 		*result = pack_float(*result, 3); /* REALM 3 */
+ 	}
+
 	POSTGIS_DEBUGF(4, "[GIST] 'penalty', union size (%.12f), original size (%.12f), penalty (%.12f)", size_union, size_orig, *result);
 
 	PG_RETURN_POINTER(result);
@@ -1545,7 +1630,7 @@ g_box_consider_split(ConsiderSplitContext *context, int dimNum,
 	POSTGIS_DEBUGF(5, "consider split: dimNum = %d, rightLower = %f, "
 		"minLeftCount = %d, leftUpper = %f, maxLeftCount = %d ",
 		dimNum, rightLower, minLeftCount, leftUpper, maxLeftCount);
-	
+
 	/*
 	 * Calculate entries distribution ratio assuming most uniform distribution
 	 * of common entries.
@@ -1718,7 +1803,7 @@ Datum gserialized_gist_picksplit_2d(PG_FUNCTION_ARGS)
 			   *intervalsUpper;
 	CommonEntry *commonEntries;
 	int			nentries;
-	
+
 	POSTGIS_DEBUG(3, "[GIST] 'picksplit' entered");
 
 	memset(&context, 0, sizeof(ConsiderSplitContext));
@@ -1741,7 +1826,7 @@ Datum gserialized_gist_picksplit_2d(PG_FUNCTION_ARGS)
 		else
 			adjustBox(&context.boundingBox, box);
 	}
-	
+
 	POSTGIS_DEBUGF(4, "boundingBox is %s", box2df_to_string(
 														&context.boundingBox));
 
@@ -1906,7 +1991,7 @@ Datum gserialized_gist_picksplit_2d(PG_FUNCTION_ARGS)
 	 */
 
 	POSTGIS_DEBUGF(4, "split direction: %d", context.dim);
-	
+
 	/* Allocate vectors for results */
 	v->spl_left = (OffsetNumber *) palloc(nentries * sizeof(OffsetNumber));
 	v->spl_right = (OffsetNumber *) palloc(nentries * sizeof(OffsetNumber));
@@ -1994,7 +2079,7 @@ Datum gserialized_gist_picksplit_2d(PG_FUNCTION_ARGS)
 			PLACE_RIGHT(box, i);
 		}
 	}
-	
+
 	POSTGIS_DEBUGF(4, "leftBox is %s", box2df_to_string(leftBox));
 	POSTGIS_DEBUGF(4, "rightBox is %s", box2df_to_string(rightBox));
 
@@ -2055,9 +2140,9 @@ Datum gserialized_gist_picksplit_2d(PG_FUNCTION_ARGS)
 	}
 	v->spl_ldatum = PointerGetDatum(leftBox);
 	v->spl_rdatum = PointerGetDatum(rightBox);
-	
+
 	POSTGIS_DEBUG(4, "[GIST] 'picksplit' completed");
-	
+
 	PG_RETURN_POINTER(v);
 }
 
@@ -2266,7 +2351,7 @@ Datum gserialized_gist_picksplit_2d(PG_FUNCTION_ARGS)
 	{
 		float sizeLR, sizeBT;
 		BOX2DF interLR, interBT;
-		
+
 		if ( box2df_intersection(unionL, unionR, &interLR) == FALSE )
 			sizeLR = 0.0;
 		else
@@ -2276,7 +2361,7 @@ Datum gserialized_gist_picksplit_2d(PG_FUNCTION_ARGS)
 			sizeBT = 0.0;
 		else
 			sizeBT = box2df_size(&interBT);
-		
+
 		if (sizeLR < sizeBT)
 			direction = 'x';
 		else
@@ -2313,9 +2398,9 @@ Datum gserialized_gist_picksplit_2d(PG_FUNCTION_ARGS)
 		v->spl_ldatum = PointerGetDatum(unionB);
 		v->spl_rdatum = PointerGetDatum(unionT);
 	}
-	
+
 	POSTGIS_DEBUG(4, "[GIST] 'picksplit' completed");
-	
+
 	PG_RETURN_POINTER(v);
 }
 
