@@ -2075,36 +2075,7 @@ Datum ST_MakeEnvelope(PG_FUNCTION_ARGS)
 		srid = PG_GETARG_INT32(4);
 	}
 
-	pa = (POINTARRAY**)palloc(sizeof(POINTARRAY**));
-	pa[0] = ptarray_construct_empty(0, 0, 5);
-
-	/* 1st point */
-	p.x = x1;
-	p.y = y1;
-	ptarray_append_point(pa[0], &p, LW_TRUE);
-
-	/* 2nd point */
-	p.x = x1;
-	p.y = y2;
-	ptarray_append_point(pa[0], &p, LW_TRUE);
-
-	/* 3rd point */
-	p.x = x2;
-	p.y = y2;
-	ptarray_append_point(pa[0], &p, LW_TRUE);
-
-	/* 4th point */
-	p.x = x2;
-	p.y = y1;
-	ptarray_append_point(pa[0], &p, LW_TRUE);
-
-	/* 5th point */
-	p.x = x1;
-	p.y = y1;
-	ptarray_append_point(pa[0], &p, LW_TRUE);
-
-	poly = lwpoly_construct(srid, NULL, 1, pa);
-	lwgeom_add_bbox(lwpoly_as_lwgeom(poly));
+	poly = lwpoly_construct_envelope(srid, x1, y1, x2, y2);
 
 	result = geometry_serialize(lwpoly_as_lwgeom(poly));
 	lwpoly_free(poly);
@@ -2661,15 +2632,21 @@ Datum ST_RemoveRepeatedPoints(PG_FUNCTION_ARGS);
 PG_FUNCTION_INFO_V1(ST_RemoveRepeatedPoints);
 Datum ST_RemoveRepeatedPoints(PG_FUNCTION_ARGS)
 {
-	GSERIALIZED *g_in = PG_GETARG_GSERIALIZED_P_COPY(0);
+	GSERIALIZED *g_in = PG_GETARG_GSERIALIZED_P(0);
+	int type = gserialized_get_type(g_in);
 	GSERIALIZED *g_out;
-	LWGEOM *lwgeom_in = lwgeom_from_gserialized(g_in);
+	LWGEOM *lwgeom_in = NULL;
 	LWGEOM *lwgeom_out = NULL;
 	double tolerance = 0.0;
 
+	/* Don't even start to think about points */
+	if ( type == POINTTYPE )
+		PG_RETURN_POINTER(g_in);
+
 	if ( PG_NARGS() > 1 && ! PG_ARGISNULL(1) )
 		tolerance = PG_GETARG_FLOAT8(1);
 
+	lwgeom_in = lwgeom_from_gserialized(g_in);
 	lwgeom_out = lwgeom_remove_repeated_points(lwgeom_in, tolerance);
 	g_out = geometry_serialize(lwgeom_out);
 
